#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <zlib.h>
#include <curl/curl.h>


extern char *fbsdmon_url(char *path)
{
    char *prefix = getenv("FBSDMON_URL");
    if(NULL != prefix) prefix = strdup(prefix);
    else prefix = strdup("http://fbsdmon.org/");

    char *url = malloc(strlen(prefix) + strlen(path) + 1);
    bzero(url, 1);
    strcat(url, prefix);
    strcat(url, path);

    free(prefix);
    return url;
}

extern char *useragent()
{
    char *ua;
    asprintf(&ua, "fbsdmon %1.2f", VERSION);
    return ua;
}

extern char *uname(char *param)
{
    char command[80], temp[80];
    sprintf(command, "uname %s", param);

    FILE *UNAME = popen(command, "r");
    fgets(temp, 80, UNAME);
    pclose(UNAME);

    temp[strlen(temp)-1] = '\0'; // cut \n from the end
    return strdup(temp);
}

extern char *datetime()
{
    static char *dt_s = NULL; 

    if(dt_s == NULL) {
        const time_t dt_t = time(NULL);
	dt_s = ctime(&dt_t);
    }

    return dt_s;
}

extern char *gz_data(char *data)
{
    char *fn = strdup("/tmp/fbsdmon.XXXXXX");
    int fd = mkstemp(fn);

    gzFile gz = gzdopen(fd, "w");
    gzputs(gz, data);
    gzclose(gz);

    return fn;
}

extern void send_file(char *fn)
{
    CURL *curl = curl_easy_init();
    char *url = fbsdmon_url("submit");
    char *ua = useragent();
    struct curl_slist *headers = NULL;

    FILE *file = fopen(fn, "r");
    struct stat *st = malloc(sizeof(struct stat));
    char *content_length;

    fstat(fileno(file), st);
    asprintf(&content_length, "Content-Length: %lld", st->st_size);

    headers = curl_slist_append(headers, "Content-Type: application/octet-stream");
    headers = curl_slist_append(headers, content_length);

    curl_easy_setopt(curl, CURLOPT_USERAGENT, ua);
    curl_easy_setopt(curl, CURLOPT_URL, url);
    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, NULL);
    curl_easy_setopt(curl, CURLOPT_READDATA, file);

    if(NULL != getenv("FBSDMON_URL")) {
        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
    }

    curl_easy_perform(curl);

    fclose(file);
    free(st);
    free(content_length);

    curl_easy_cleanup(curl);
    free(ua);
    free(url);
    curl_slist_free_all(headers);
}

extern void archive_stats(char *json)
{
    FILE *file = fopen("/var/db/fbsdmon/STATS", "w");
    fputs(json, file);
    fputs("\n", file);
    fclose(file);
}

extern void save_submit_date()
{
    FILE *file = fopen("/var/db/fbsdmon/DATE", "w");
    fputs(datetime(), file);
    fclose(file);
}
