#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/param.h>
#include <libutil.h>
#include <errno.h>

#include "info.h"
#include "json.h"
#include "uuid.h"
#include "io.h"


struct pidfh *pfh;

struct cmdl_t
{
    int sleep:1;
    int daemon:1;
    int test:1;
};


void exit_program(int sig)
{
    int status = pidfile_remove(pfh);
    exit(status);
}

int fbsdmon_test()
{
    return 0;
}

void program()
{
    char *json = get_json_root();
    char *fn = gz_data(json);

    send_file(fn);
    save_submit_date();
    archive_stats(json);

    free(json);
    json_flush();
    unlink(fn);
    free(fn);
}

int main(int argc, char **argv)
{
    struct cmdl_t cmdl = {0, 0, 0};
    pid_t other_pid;

    while(--argc && argv++) {
        if(!strcmp(*argv, "-D")) cmdl.daemon = 1;
        else if(!strcmp(*argv, "-s")) cmdl.sleep = 1;
        else if(!strcmp(*argv, "-t")) cmdl.test = 1;
    }


    if(cmdl.daemon) {
        pfh = pidfile_open(NULL, 0600, &other_pid);
        if(pfh == NULL) {
            return 1;
        }
        daemon(0, 0);
        pidfile_write(pfh);

        signal(15, exit_program);
    }

    if(cmdl.sleep) {
        srandomdev();
        sleep(random() % 600);
    }

    if(cmdl.test) {
        return fbsdmon_test();
    }


    program();


    if(cmdl.daemon) {
        return pidfile_remove(pfh);
    }

    return 0;
}
